/*** Disclaimer ******************************************************************/
/*                                                                               */
/*                                                                               */
/*  THE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,                   */
/*  EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO WARRANTIES          */
/*  OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.                      */
/*  IN NO EVENT WILL THE AUTHOR OR AUTHORS BE LIABLE TO YOU FOR ANY DAMAGES,     */
/*  INCLUDING INCIDENTAL OR CONSEQUENTIAL DAMAGES, ARISING OUT OF THE USE        */
/*  OF THE CODE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.             */
/*  YOU ACKNOWLEDGE THAT YOU HAVE READ THIS LICENSE, UNDERSTAND IT AND AGREE     */
/*  TO BE BOUND BY ITS TERMS AS THE COMPLETE AND EXCLUSIVE STATEMENT OF          */
/*  THE AGREEMENT BETWEEN US, SUPERSEDING ANY PROPOSAL OR PRIOR AGREEMENT,       */
/*  ORAL OR WRITTEN, AND ANY OTHER COMMUNICATIONS BETWEEN US RELATING            */
/*  TO THE SUBJECT MATTER OF THIS LICENSE.                                       */
/*                                                                               */
/*********************************************************************************/

// www.byvac.com
// This is intended for use with the BV4221 USB to I2c
// It will take a script and send it to the specified com port. The script
// first line should contain:
// COM2, 115200
// Assuming COM2 and 115200 Baud rate are required.
//
// Compile with Borland free C compiler bcc32 like this:
// bcc32 i2com.c
//
// This is free software and accepted as-is, please make any improvements
// and send them to the contact at www.byvac.com 

#include <stdlib.h>
#include <windows.h>
#include <stdio.h>
#include <time.h>

// Global handle for communiction to serial port
HANDLE g_hCom;
int pDelay;     // used for delay between lines

// prototypes
void send_com_text(FILE *fp);
int get_next(FILE *ifp, char *buf);
int get_line(FILE *ifp, char *s);
void bvt_wait ( int ticks );
void vbt_vh(int vert, int hor);
int bvt_init(char *port, int Baud);
int com_puts(char *s);
int com_gets(char *s);
int open_com(char *pcCommPort, double baud, HANDLE *hCom);


int main(int argc, char *argv[])
{
HANDLE hCom;
char buf[258], cport[20], *bp;
FILE *fp;
int br;

    if(argc==2) 
        fp=fopen(argv[1], "r");
        if(fp) {
    
            // COM port
            get_next(fp,buf);   // get next token
            strcpy(cport,buf); // save com port name
            
            // Baud rate
            get_next(fp,buf);   // get next token
            sscanf(buf,"%d",&br); // convert to int
                
            // Delay
            get_next(fp,buf);   // get next token
            sscanf(buf,"%d",&pDelay); // convert to int

            // initialise port                        
            if(bvt_init(cport,br)) { // some error
                printf("\nError opening ComPort=%s, Baudrate=%d",cport,br);
            }

            send_com_text(fp);

            fclose(fp);
            CloseHandle(g_hCom);
            
        } else {
        printf("\nError opening file %s",argv[1]);
        }
    return  0;
}

// -----------------------------------------------------------------------------
// sends text to com port almost raw but for conveniance if it is in ""
// then it will send the byte followd by a space.
// obtains but throws away any input recieved back from BV4221
// -----------------------------------------------------------------------------
void send_com_text(FILE *fp)
{
char buf[258], out[2580], tok[30], *bp, *obp;
int quote;

    while(!get_next(fp,buf)) {
        quote=0;bp=buf;obp=out;
        *obp=0; // just in case
        while(*bp) { // parse whole of line
            if(*bp==0) break;
            if(*bp=='"') { 
                if(quote) quote=0; else quote=1; // toggle
                bp++;  // next
            }
            if(!quote) {
                // just send normal
                *(obp++)=*(bp++);  
            } else {
                sprintf(tok,"%x ",*bp);
                *obp=0; // terminate for stringcat
                strcat(out,tok);
                obp+=strlen(tok);
                bp++;
            }
                
        }
        *(obp++)=13; // need to add CR
        *obp=0;
//        printf("\nline --%s--",out);
        com_puts(out); // send it  
        com_gets(buf); // important to wait for return      
    }
}
// -----------------------------------------------------------------------------
// gets next token in file. Token spearators are ; EOL
// comments # are ignored and the next line is fetched
// -----------------------------------------------------------------------------
int get_next(FILE *ifp, char *s)
{
char *bp=s; 
int c=0, j=0;

    *bp=0;      // start at nothing
    while(c != EOF) {
        c = fgetc(ifp);
        if((c < 31)||(c==';')) break;
        if(c=='#') {
            if(j > 1) {
                while((c=fgetc(ifp)>31 && (c!=EOF))); // ignore rest of line
                // trim trailing spaces from s
                j--;
                while(*(bp+j--)==' ');
                j+=2;
                break;
            } else {
                while((c=fgetc(ifp)>31)); // start of next line                
                j=0; // reset pointer
            }    
        }
        else *(bp+j++)=c;       
    }        
    *(bp+j)=0;    
    if(c == EOF) return 1;    
    return 0;
}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Basic serial port communication without CTS handshake
//
//
//
// -----------------------------------------------------------------------------
// used for initialisation, as removing the spash screen does not assert
// RTS so some delay is needed
// -----------------------------------------------------------------------------
void bvt_wait ( int ticks )
{
  clock_t endwait;
  endwait = clock () + ticks;
  while (clock() < endwait) {}
}

// -----------------------------------------------------------------------------
// Initialsise bv4221 device
// -----------------------------------------------------------------------------
int bvt_init(char *port, int Baud)
{
HANDLE hCom;
char s[5];

    if((open_com(port,(double)Baud,&hCom))) return 1; // error
    g_hCom=hCom;    // set global
    // send initial CR
    s[0]=13;s[1]=0;
    com_puts(s);
    bvt_wait(100);
    return 0;
}

// -----------------------------------------------------------------------------
// write string to open com, no handshake
// -----------------------------------------------------------------------------
int com_puts(char *s)
{
unsigned long nbr;

    WriteFile(g_hCom,s,strlen(s),&nbr,0);     
    return nbr;
}

// -----------------------------------------------------------------------------
// This will get a line fo text from the BV4221 which is always terminated
// by '>'
// -----------------------------------------------------------------------------
int com_gets(char *s)
{
unsigned long nbr;
char c[2], *bp, max=1;

    bp=s;
    while(1) {
        if(!ReadFile(g_hCom,c,1,&nbr,0)) {
            printf("\nError on reading com");
            return 0;
        }
        if(c[0]=='>') break;
        if(!max++) return 0;
        *(bp++)=c[0];
    }
    *bp=0;
    bvt_wait(pDelay);  // very short wait after recieving the '>'
    return 1;
}        

// -----------------------------------------------------------------------------
// Open a given com port and return , parameters fixed in
// this code for now as
// 8+1 stop
// CTS cannot be checked here for handshaking, it must be done as in putc
// or as an event.
// -----------------------------------------------------------------------------
int open_com(char *pcCommPort, double baud, HANDLE *hCom)
{
DCB dcb;
BOOL fSuccess;

*hCom =  CreateFile( pcCommPort,
        GENERIC_READ | GENERIC_WRITE,
        0, // must be opened with exclusive-access
        NULL, // no security attributes
        OPEN_EXISTING, // must use OPEN_EXISTING
        0, // not overlapped I/O
        NULL // hTemplate must be NULL for comm devices
    );

    if (*hCom == INVALID_HANDLE_VALUE) {    // Handle the error.
        printf ("CreateFile failed with error %d.\n", GetLastError());
        return (1);
    }

    // Build on the current configuration, and skip setting the size
    // of the input and output buffers with SetupComm.
    fSuccess = GetCommState(*hCom, &dcb);

    if (!fSuccess) {
    // Handle the error.
        printf ("GetCommState failed with error %d.\n", GetLastError());
        return (2);
    }

    // Fill in DCB: 57,600 bps, 8 data bits, no parity, and 1 stop bit.

    dcb.BaudRate = baud; // set the baud rate
    dcb.ByteSize = 8; // data size, xmit, and rcv
    dcb.Parity = NOPARITY; // no parity bit
    dcb.StopBits = ONESTOPBIT; // one stop bit
//    dcb.fOutxCtsFlow = 3;
//    dcb.fRtsControl = RTS_CONTROL_ENABLE;
//    dcb.fDtrControl = DTR_CONTROL_ENABLE;

    fSuccess = SetCommState(*hCom, &dcb);

    if (!fSuccess) { // Handle the error.
        printf ("SetCommState failed with error %d.\n", GetLastError());
        return (3);
    }
    
    return 0;

}
